package com.teamdev.xpcom.mozilla.demo.components;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.event.ChangeEvent;

import com.teamdev.jxbrowser.WebBrowser;
import com.teamdev.jxbrowser.WebBrowserFactory;
import com.teamdev.jxbrowser.event.LocationEvent;
import com.teamdev.jxbrowser.event.RequestAdapter;
import com.teamdev.jxbrowser.event.RequestListener;
import com.teamdev.jxbrowser.event.StateEvent;
import com.teamdev.jxbrowser.printing.PaperSize;
import com.teamdev.jxbrowser.printing.PrintSettings;
import com.teamdev.jxbrowser.printing.WebBrowserPrinting;
import com.teamdev.xpcom.Xpcom;
import com.teamdev.xpcom.mozilla.demo.components.PrintPreviewToolBar.OrientationAction;
import com.teamdev.xpcom.mozilla.demo.components.PrintPreviewToolBar.ToolBarActionHandler;

/**
 * This class used for print preview web page using JxBrowser. NOTE: The
 * JxBrowser print preview don't based on the default AWT print preview
 * technology. It's a Mozilla based native print preview. Do not try mix a AWT
 * and Mozilla print preview. Also this class can't be used at Mac OS, because
 * Mozilla don't implement print preview functionality at MacOS X. The FireFox
 * uses the third-party application for this propouse.
 * 
 * @author Victor Gubin
 */
public class PrintPreviewPanel extends JPanel
{
    private static final long serialVersionUID = -2364402827708391282L;

    /**
     * An parent frame for insert this panel
     */
    private final JFrame frame;

    /**
     * The browser component for switching to print preview mode
     */
    private WebBrowser browser;
    /**
     * Entry point for print preview functionality
     */
    private WebBrowserPrinting printAPI;
    /**
     * The print configuration, such as an printer name, paper size, margins,
     * headers/footers etc.
     */
    private PrintSettings settings;
    /**
     * The Swing tool-bar for manipulating the print preview
     */
    private final PrintPreviewToolBar controlToolBar;

    private JPanel preparingPanel;
    private JLabel preparingLabel = new JLabel(
            "Preparing browser component to print preview mode. Wait...");
    private Runnable closeAction;

    /**
     * Construct new print preview panel
     * 
     * @param frame
     *            the frame whether print preview should be inserted instead
     *            components that is already inserted into this frame.
     * @param parentBrowser
     *            the web browser that contains document that s should be print
     *            previewed
     * @param closeAction
     *            action on closing printpreview mode.  
     */

    public PrintPreviewPanel(final JFrame frame, final WebBrowser parentBrowser, Runnable closeAction)
    {
        super(new BorderLayout());

        this.closeAction = closeAction;
        this.frame = frame;
        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                JMenuBar menu = frame.getJMenuBar();
                if(menu != null) menu.setVisible(false);
            }
        });

        controlToolBar = createControlToolBar();
        controlToolBar.setEnabled(false);
        add(controlToolBar, BorderLayout.NORTH);

        preparingPanel = new JPanel();
        preparingPanel.add(preparingLabel, BorderLayout.CENTER);
        add(preparingPanel, BorderLayout.CENTER);

        new Thread(new Runnable()
        {
            public void run()
            {
                browser = WebBrowserFactory.getInstance().createBrowser();
                configurePreviewBrowser(parentBrowser);
                remove(preparingPanel);
                add(browser.getComponent(), BorderLayout.CENTER);
                controlToolBar.setEnabled(true);
                validate();
                resizeBrowserComponent(browser);
            }
        }).start();
    }

    /**
     * Configure the browser that will be used for print preview
     * 
     * @param parentBrowser
     *            the web browser that contains document that s should be print
     *            previewed
     */
    private void configurePreviewBrowser(final WebBrowser parentBrowser)
    {
        final AtomicBoolean ready = new AtomicBoolean(false);
        RequestListener navigateListener = new RequestAdapter()
        {
            String userRequestURI;

            @Override
            public void locationChanged(LocationEvent event)
            {
                userRequestURI = event.getRequestUrlName();
            }

            @Override
            public void stateChanged(StateEvent event)
            {
                boolean documentCompleted = userRequestURI.equals(event
                        .getRequestUrlName());
                if (event.isLoadingCompleted() && documentCompleted)
                {
                    ready.set(true);
                    Xpcom.unlock();
                }
            }
        };

        browser.addRequestListener(navigateListener);
        browser.getComponent().setEnabled(false);
        browser.getComponent().setVisible(false);
        if (!parentBrowser.getLocationURL().equals("about:blank"))
        {
            browser.navigate(parentBrowser.getLocationURL());
        }
        else
        {
            ready.set(true);
        }
        // Wait for navigation finish
        if (!ready.get())
        {
            while (!ready.get())
            {
                Xpcom.lock();
            }
        }
        browser.removeRequestListener(navigateListener);
        browser.setAttributes(parentBrowser.getAttributes());
        printAPI = browser.getPrinting();
        settings = printAPI.getCurrentPrintSettings();
        settings.setPaperSize(PaperSize.A4);
        printAPI.setPrintPreviewMode(true);
        controlToolBar.setPageCount(printAPI.getPrintPreviewPageNumbers());
        browser.getComponent().setEnabled(true);
        browser.getComponent().setVisible(true);
    }

    /**
     * Handler for Print button in toll bar
     */
    private class PrintBtnHandler implements ToolBarActionHandler<ActionEvent>
    {
        public void performed(ActionEvent event)
        {
            new PrintDialog(frame, browser);
        }
    }

    /**
     * Handler for spinner in tool bar
     */
    private class ScrollHanlder implements ToolBarActionHandler<ChangeEvent>
    {
        public void performed(ChangeEvent event)
        {
            SpinnerNumberModel spinner = (SpinnerNumberModel) event.getSource();
            printAPI.printPreviewNavigate(
                    WebBrowserPrinting.PRINTPREVIEW_GOTO_PAGENUM,
                    (Integer) spinner.getValue());
        }
    }

    /**
     * Handler for "Portrait" and "Landscape" radio-buttons in tool bar
     */
    private class OrientationHandler implements
            ToolBarActionHandler<OrientationAction>
    {
        public void performed(OrientationAction event)
        {
            settings.setOrientation(event.getOrientation());
            printAPI.updatePrintPreview();
            controlToolBar.setPageCount(printAPI.getPrintPreviewPageNumbers());
            resizeBrowserComponent(browser);
        }
    }

    /**
     * Handler for Paper size ComboBox in tool bar
     */
    private class PaperSizeHandler implements ToolBarActionHandler<ActionEvent>
    {
        public void performed(ActionEvent event)
        {
            JComboBox comboBox = (JComboBox) event.getSource();
            PaperSize size = (PaperSize) comboBox.getSelectedItem();
            settings.setPaperSize(size);
            printAPI.updatePrintPreview();
            controlToolBar.setPageCount(printAPI.getPrintPreviewPageNumbers());
            resizeBrowserComponent(browser);
        }
    }

    /**
     * Handler for backgrounds check box in tool bar
     */
    private class BackroundsHandler implements
            ToolBarActionHandler<ActionEvent>
    {
        public void performed(ActionEvent event)
        {
            boolean bacgrounds = ((JCheckBox) event.getSource()).isSelected();
            settings.setPrintBackgroundColors(bacgrounds);
            settings.setPrintBackgroundImages(bacgrounds);
            printAPI.updatePrintPreview();
            controlToolBar.setPageCount(printAPI.getPrintPreviewPageNumbers());
            resizeBrowserComponent(browser);
        }
    }

    private void resizeBrowserComponent(final WebBrowser browser)
    {
        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                Dimension dimension = browser.getComponent().getSize();
                browser.getComponent().setSize((int) dimension.getWidth(),
                        (int) (dimension.getHeight() + 1));
                browser.getComponent().setSize(dimension);

            }
        });
    }

    /**
     * Handler for Close button in tool bar
     */
    private class CloseHandler implements ToolBarActionHandler<ActionEvent>
    {
        public void performed(ActionEvent event)
        {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    JMenuBar menu = frame.getJMenuBar();
                    if (null != menu) menu.setVisible(true);
                }
            });

            closeAction.run();
        }
    }

    /**
     * Create and configure ControlToolBar
     * @return panel with some controls for printpreview mode
     */
    private PrintPreviewToolBar createControlToolBar()
    {
        final PrintPreviewToolBar controlToolBar = new PrintPreviewToolBar();
        controlToolBar.setPrintBtnHandler(new PrintBtnHandler());
        controlToolBar.setScrollPageHandler(new ScrollHanlder());
        controlToolBar.setOrientationChangedHandler(new OrientationHandler());
        controlToolBar.setPaperSizeChangedHandler(new PaperSizeHandler());
        controlToolBar.setBackgroudsFlagChangedHandler(new BackroundsHandler());
        controlToolBar.setCloseBtnHandler(new CloseHandler());
        return controlToolBar;
    }

    public static void main(String[] args) throws ClassNotFoundException, UnsupportedLookAndFeelException, IllegalAccessException, InstantiationException
    {
        JPopupMenu.setDefaultLightWeightPopupEnabled(false);
        if (!Xpcom.isLinux())
        {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }
        else
        {
            UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
        }
        Xpcom.initialize();
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                final WebBrowser browser = WebBrowserFactory.getInstance().createBrowser();
                final JFrame frame = new JFrame("PrintPreviewTestFrame");

                final CardLayout contentLayout = new CardLayout();
                final JPanel contentPanel = new JPanel(contentLayout);
                JPanel browserPanel = new JPanel(new BorderLayout());
                final JPanel previewPanel = new JPanel(new BorderLayout());
                contentPanel.add("browserPanel", browserPanel);
                contentPanel.add("previewPanel", previewPanel);
                browserPanel.add(new JButton(new AbstractAction("PrintPreview") {

                    public void actionPerformed(ActionEvent event)
                    {
                        browser.stop();
                        JPanel printPreviewPanel = new PrintPreviewPanel(frame, browser,
                                        new Runnable() {
                                            public void run()
                                            {
                                                previewPanel.removeAll();
                                                contentLayout.next(contentPanel);
                                            }
                                        });
                        previewPanel.add(printPreviewPanel,BorderLayout.CENTER);
                        contentLayout.next(contentPanel);
                    }

                }), BorderLayout.NORTH);
                browserPanel.add(browser.getComponent(), BorderLayout.CENTER);

                frame.getContentPane().setLayout(new BorderLayout());
                frame.getContentPane().add(contentPanel, BorderLayout.CENTER);
                frame.setSize(800, 600);
                frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
                frame.setLocationRelativeTo(null);
                frame.setVisible(true);

                browser.navigate("http://www.google.com");
            }
        });
    }
}
